@@ -18,7 +18,6 @@ from mathics.builtin.base import (
     BoxConstruct,
     BoxConstructError,
     Operator,
-    Predefined,
 )
 from mathics.builtin.tensors import get_dimensions
 from mathics.builtin.comparison import expr_min
@@ -51,35 +50,6 @@ from mathics.core.evaluation import Message as EvaluationMessage
 MULTI_NEWLINE_RE = re.compile(r"\n{2,}")
 
 
-class UseSansSerif(Predefined):
-    """
-    <dl>
-      <dt>'$UseSansSerif'
-      <dd>controls whether the Web interfaces use a Sans-Serif font.
-    </dl>
-
-    When set True, the output in MathMLForm uses SansSerif fonts instead
-    of the standard fonts.
-
-    X> $UseSansSerif
-     = True
-    X> $UseSansSerif = False
-
-    """
-
-    context = "System`"
-    name = "$UseSansSerif"
-    attributes = ("Unprotected",)
-    value = True
-
-    rules = {"$UseSansSerif": str(value)}
-
-    messages = {}
-
-    def evaluate(self, evaluation):
-        return Integer(self.value)
-
-
 class Format(Builtin):
     """
     <dl>
@@ -764,7 +734,7 @@ class Row(Builtin):
         else:
             result = []
             for index, item in enumerate(items):
-                if index > 0 and not sep.same(String("")):
+                if index > 0 and not sep.sameQ(String("")):
                     result.append(sep)
                 result.append(MakeBoxes(item, f))
             return RowBox(Expression(SymbolList, *result))
@@ -1516,9 +1486,18 @@ class Quiet(Builtin):
         <dd>evaluates $expr$, with messages $off$ being suppressed, but messages $on$ being displayed.
     </dl>
 
+    Evaluate without generating messages:
+    >> Quiet[1/0]
+     = ComplexInfinity
+
+    Same as above:
+    >> Quiet[1/0, All]
+     = ComplexInfinity
+
     >> a::b = "Hello";
     >> Quiet[x+x, {a::b}]
      = 2 x
+
     >> Quiet[Message[a::b]; x+x, {a::b}]
      = 2 x
 
@@ -1527,7 +1506,7 @@ class Quiet(Builtin):
      : Hello
      = 2 x
 
-    >> Quiet[expr, All, All]
+    #> Quiet[expr, All, All]
      : Arguments 2 and 3 of Quiet[expr, All, All] should not both be All.
      = Quiet[expr, All, All]
     >> Quiet[x + x, {a::b}, {a::b}]
@@ -2096,10 +2075,10 @@ class MathMLForm(Builtin):
 
         # mathml = '<math><mstyle displaystyle="true">%s</mstyle></math>' % xml
         # #convert_box(boxes)
-        query = evaluation.parse("System`$UseSansSerif")
+        query = evaluation.parse("Settings`$UseSansSerif")
         usesansserif = query.evaluate(evaluation).to_python()
         if not is_a_picture:
-            if usesansserif:
+            if isinstance(usesansserif, bool) and usesansserif:
                 xml = '<mstyle mathvariant="sans-serif">%s</mstyle>' % xml
 
         mathml = '<math display="block">%s</math>' % xml  # convert_box(boxes)
@@ -2294,20 +2273,20 @@ class _NumberForm(Builtin):
 
     def check_DigitBlock(self, value, evaluation):
         py_value = value.get_int_value()
-        if value.same(Symbol("Infinity")):
+        if value.sameQ(Symbol("Infinity")):
             return [0, 0]
         elif py_value is not None and py_value > 0:
             return [py_value, py_value]
         elif value.has_form("List", 2):
             nleft, nright = value.leaves
             py_left, py_right = nleft.get_int_value(), nright.get_int_value()
-            if nleft.same(Symbol("Infinity")):
+            if nleft.sameQ(Symbol("Infinity")):
                 nleft = 0
             elif py_left is not None and py_left > 0:
                 nleft = py_left
             else:
                 nleft = None
-            if nright.same(Symbol("Infinity")):
+            if nright.sameQ(Symbol("Infinity")):
                 nright = 0
             elif py_right is not None and py_right > 0:
                 nright = py_right
@@ -2319,7 +2298,7 @@ class _NumberForm(Builtin):
         return evaluation.message(self.get_name(), "dblk", value)
 
     def check_ExponentFunction(self, value, evaluation):
-        if value.same(Symbol("Automatic")):
+        if value.sameQ(Symbol("Automatic")):
             return self.default_ExponentFunction
 
         def exp_function(x):
@@ -2328,7 +2307,7 @@ class _NumberForm(Builtin):
         return exp_function
 
     def check_NumberFormat(self, value, evaluation):
-        if value.same(Symbol("Automatic")):
+        if value.sameQ(Symbol("Automatic")):
             return self.default_NumberFormat
 
         def num_function(man, base, exp, options):
@@ -2355,9 +2334,9 @@ class _NumberForm(Builtin):
         return result
 
     def check_SignPadding(self, value, evaluation):
-        if value.same(Symbol("True")):
+        if value.sameQ(Symbol("True")):
             return True
-        elif value.same(Symbol("False")):
+        elif value.sameQ(Symbol("False")):
             return False
         return evaluation.message(self.get_name(), "opttf", value)
 
