@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-from .helper import session
+from .helper import evaluate
 
 import sys
 import time
@@ -7,7 +7,7 @@ import time
 if sys.platform not in ("win32",):
     def test_datentime():
         str_expr = "TimeConstrained[1+2; TimeRemaining[], 0.9]"
-        result = session.evaluate(str_expr)
+        result = evaluate(str_expr)
         assert result is None or 0 < result.to_python() < 9
 
 
@@ -16,9 +16,9 @@ if sys.platform not in ("darwin", "win32",):
     def test_timeconstrained1():
         #
         str_expr1 = "a=1.; TimeConstrained[Do[Pause[.1];a=a+1,{1000}],1]"
-        result = session.evaluate(str_expr1)
+        result = evaluate(str_expr1)
         str_expected = "$Aborted"
-        expected = session.evaluate(str_expected)
+        expected = evaluate(str_expected)
         assert result == expected
         time.sleep(1)
-        assert session.evaluate("a").to_python() == 10
+        assert evaluate("a").to_python() == 10
