@@ -1,13 +1,5 @@
 # -*- coding: utf-8 -*-
-from .helper import session, check_evaluation
-
-import math
-import sympy
-import sys
-from mathics.core.definitions import Definitions
-from mathics.core.evaluation import Evaluation
-import pytest
-
+from .helper import check_evaluation
 
 def test_to_infinity():
     for str_expr, str_expected, message in (
